@@ -41,7 +41,7 @@ sendto_callback (NautilusMenuItem *item,
 {
 	GList            *files, *scan;
 	NautilusFileInfo *file;
-	gchar            *uri, *path;
+	gchar            *uri;
 	GString          *cmd;
 
 	files = g_object_get_data (G_OBJECT (item), "files");
@@ -69,9 +69,7 @@ nautilus_nste_get_file_items (NautilusMenuProvider *provider,
 {
 	GList    *items = NULL;
 	GList    *scan;
-	gboolean  can_write = FALSE;
 	gboolean  one_item;
-	gboolean  one_archive = FALSE;
 	NautilusMenuItem *item;
 
 	if (files == NULL)
@@ -97,32 +95,23 @@ nautilus_nste_get_file_items (NautilusMenuProvider *provider,
 					       _("Send to..."),
 					       _("Send file by mail, instant message..."),
 					       NULL);
-		g_signal_connect (item, 
-				  "activate",
-				  G_CALLBACK (sendto_callback),
-			  	provider);
-		g_object_set_data_full (G_OBJECT (item), 
-					"files",
-					nautilus_file_info_list_copy (files),
-					(GDestroyNotify) nautilus_file_info_list_free);
-
-		items = g_list_append (items, item);
-	}else{
+	} else {
 		item = nautilus_menu_item_new ("NautilusNste::sendto",
 					       _("Send to..."),
 					       _("Send files by mail, instant message..."),
 					       NULL);
-		g_signal_connect (item, 
-				  "activate",
-				  G_CALLBACK (sendto_callback),
-			  	provider);
-		g_object_set_data_full (G_OBJECT (item), 
-					"files",
-					nautilus_file_info_list_copy (files),
-					(GDestroyNotify) nautilus_file_info_list_free);
-
-		items = g_list_append (items, item);		
 	}
+  g_signal_connect (item, 
+      "activate",
+      G_CALLBACK (sendto_callback),
+      provider);
+  g_object_set_data_full (G_OBJECT (item), 
+      "files",
+      nautilus_file_info_list_copy (files),
+      (GDestroyNotify) nautilus_file_info_list_free);
+
+  items = g_list_append (items, item);
+
 	return items;
 }
 
